package com.dzeio.openhealth.extensions

import android.Manifest
import android.app.Activity
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.util.Log
import androidx.core.app.ActivityCompat
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.dzeio.openhealth.data.weight.Weight
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.fitness.Fitness
import com.google.android.gms.fitness.FitnessOptions
import com.google.android.gms.fitness.data.DataType
import com.google.android.gms.fitness.request.DataReadRequest
import java.text.DateFormat
import java.util.Calendar
import java.util.Date
import java.util.TimeZone
import java.util.concurrent.TimeUnit

class GoogleFit() : Extension() {
    companion object {
        const val TAG = "GoogleFitConnector"
    }

    private lateinit var activity: Activity

    override val id = "GoogleFit"
    override val name = "Google Fit"

    override fun init(activity: Activity): Array<Data> {
        this.activity = activity
        return arrayOf(
            Data.WEIGHT
        )
    }

    override fun getStatus(): String {
        return if (isConnected()) "Connected" else "Not Connected"
    }

    override fun isAvailable(): Boolean {
        return true
    }

    override fun isConnected(): Boolean =
        GoogleSignIn.hasPermissions(getGoogleAccount(), fitnessOptions)

    private val fitnessOptions = FitnessOptions.builder()
        .addDataType(DataType.TYPE_WEIGHT)
//        .addDataType(DataType.TYPE_STEP_COUNT_CUMULATIVE)
//        .addDataType(DataType.TYPE_CALORIES_EXPENDED)
        .build()

//    private fun checkPermissionsAndRun(data: Data) {
//        if (permissionApproved()) {
//            signIn(data)
//        } else {
//            Log.d(TAG, "Asking for permission")
//            // Ask for permission
//            ActivityCompat.requestPermissions(
//                activity,
//                arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
//                data.ordinal
//            )
//        }
//    }

    private fun permissionApproved(): Boolean =
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            PackageManager.PERMISSION_GRANTED == ActivityCompat.checkSelfPermission(
                activity,
                Manifest.permission.ACCESS_FINE_LOCATION
            )
        } else {
            true
        }

    private val connectLiveData: MutableLiveData<States> = MutableLiveData(States.WIP)

    override fun connect(): LiveData<States> {

        if (!permissionApproved()) {
            ActivityCompat.requestPermissions(
                activity,
                arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                87531
            )
            return connectLiveData
        }

        if (isConnected()) {
            connectLiveData.value = States.DONE
        } else {
            Log.d("GoogleFitImporter", "Signing In")
            GoogleSignIn.requestPermissions(
                activity,
                124887,
                getGoogleAccount(), fitnessOptions
            )
        }
        return connectLiveData
    }

//    private fun signIn(data: Data) {
//        if (isConnected()) {
//            startImport(data)
//        } else {
//            Log.d("GoogleFitImporter", "Signing In")
//            GoogleSignIn.requestPermissions(
//                activity,
//                data.ordinal,
//                getGoogleAccount(), fitnessOptions
//            )
//        }
//    }

    private fun getGoogleAccount() = GoogleSignIn.getAccountForExtension(activity, fitnessOptions)

    private val timeRange by lazy {
        val calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"))
        calendar.time = Date()
        val endTime = calendar.timeInMillis

        // Set year to 2013 to be sure to get data from when Google Fit Started to today
        calendar.set(Calendar.YEAR, 2013)
        val startTime = calendar.timeInMillis
        return@lazy arrayOf(startTime, endTime)
    }

    private fun startImport(data: Data) {
        Log.d("GoogleFitImporter", "Importing for ${data.name}")

        val dateFormat = DateFormat.getDateInstance()
        Log.i(TAG, "Range Start: ${dateFormat.format(timeRange[0])}")
        Log.i(TAG, "Range End: ${dateFormat.format(timeRange[1])}")

        var type = DataType.TYPE_WEIGHT
        var timeUnit = TimeUnit.MILLISECONDS

        when (data) {
            Data.STEPS -> {
                type = DataType.TYPE_STEP_COUNT_CUMULATIVE
            }
            else -> {}
        }

        runRequest(
            DataReadRequest.Builder()
                .read(type)
                .setTimeRange(timeRange[0], timeRange[1], timeUnit)
                .build(),
            data
        )
    }

    private fun runRequest(request: DataReadRequest, data: Data) {
        Fitness.getHistoryClient(
            activity,
            GoogleSignIn.getAccountForExtension(activity, fitnessOptions)
        )
            .readData(request)
            .addOnSuccessListener { response ->
                Log.d(
                    TAG,
                    "Received response! ${response.dataSets.size} ${response.buckets.size} ${response.status}"
                )
                for (dataSet in response.dataSets) {
                    Log.i(
                        TAG,
                        "Data returned for Data type: ${dataSet.dataType.name} ${dataSet.dataPoints.size} ${dataSet.dataSource.toDebugString()}"
                    )
                    dataSet.dataPoints.forEach { dp ->

                        // Global
                        Log.i(TAG, "Importing Data point:")
                        Log.i(TAG, "\tType: ${dp.dataType.name}")
                        Log.i(
                            TAG,
                            "\tStart: ${Date(dp.getStartTime(TimeUnit.SECONDS) * 1000L).toLocaleString()}"
                        )
                        Log.i(
                            TAG,
                            "\tEnd: ${Date(dp.getEndTime(TimeUnit.SECONDS) * 1000L).toLocaleString()}"
                        )

                        // Field Specifics
                        for (field in dp.dataType.fields) {
                            Log.i(TAG, "\tField: ${field.name} Value: ${dp.getValue(field)}")
                            when (data) {
                                Data.WEIGHT -> {
                                    val weight = Weight()
                                    weight.timestamp = dp.getStartTime(TimeUnit.MILLISECONDS)
                                    weight.weight = dp.getValue(field).asFloat()
                                    val list = weightLiveData.value?.list?.toMutableList()
                                        ?: ArrayList()
                                    list.add(weight)
                                    weightLiveData.value =

                                        ImportState(States.WIP, list)
                                }
                                else -> {}
                            }
                        }
                    }
                    when (data) {
                        Data.WEIGHT -> {
                            weightLiveData.value =
                                ImportState(
                                    States.DONE,
                                    weightLiveData.value?.list
                                        ?: ArrayList()
                                )
                        }
                        else -> {}
                    }
                }
            }
            .addOnFailureListener { e ->
                Log.e(TAG, "There was an error reading data from Google Fit", e)
            }
    }

    /**
     * Currently not usable
     */
    override fun onRequestPermissionResult(
        requestCode: Int,
        permission: Array<String>,
        grantResult: IntArray
    ) {
        connect()
        // signIn(Data.values()[requestCode])
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (requestCode == 0) {
            return
        }
        connectLiveData.value = States.DONE
        // signIn(Data.values()[requestCode])
    }

    private lateinit var weightLiveData: MutableLiveData<ImportState<Weight>>

    override fun importWeight(): LiveData<ImportState<Weight>> {

        weightLiveData = MutableLiveData(
            ImportState(
                States.WIP
            )
        )

        startImport(Data.WEIGHT)

//        checkPermissionsAndRun(Data.WEIGHT)

        return weightLiveData
    }
}
